import numpy
from dataStack import dataStack
from LamDelosmeVarianceFunction import LamDelosmeVarianceFunction
from LamDelosmeSeriesFunction import LamDelosmeSeriesFunction

class LamDelosmeTemperatureSchedule:
	def __init__(self,initialBurnIter,tauIter,La,Lb,Lambda,sTemp0):
		self.tauIter = tauIter
		self.initialBurnIter = initialBurnIter
		self.LIter = 0
		self.La = La
		self.Lb = Lb
		self.sTemp = [sTemp0]
		self.muHat = dataStack(tauIter)
		self.sigmaHat = dataStack(tauIter)
		self.sTempAtUpdateSteps = dataStack(tauIter)
		self.sTempAtUpdateSteps.push(sTemp0)
		self.uAtUpdateStep = []
		self.vAtUpdateStep = []
		self.Lambda = Lambda
		#First the schedule will run at infinite temperature.
		#self.runningInitialBurn = True
		
		#Make a stack for recording the initial burn data.
		self.initialBurnData = dataStack(initialBurnIter)
		
		#Make a stack for recording the last "tau" energy values.
		self.tauData = dataStack(tauIter)
		
	def initialBurnUpdate(self,energyValue):
		self.initialBurnData.push(energyValue)
		
	def initializeParameters(self):
		C0 = numpy.mean(self.initialBurnData.Data)
		#print C0
		S0 = numpy.std(self.initialBurnData.Data)
		#print S0
		#print 'Hi Joker!'
		s0 = self.sTemp[0]
		self.A = S0**2/C0**2
		self.B = 1./C0 - self.A*s0
		self.D = S0/C0
		self.E = 1/S0 - self.D*s0
		#self.muHat.push(1./(self.A*s0 + self.B))
		self.sigmaHat.push(1./(self.D*s0 + self.E))
		self.sTemp.append(1./(2*self.sigmaHat.Data[0]))
		
	def standardUpdate(self,energyValue,rho):
		self.tauData.push(energyValue)
		sn = self.sTemp[-1]
		self.muHat.push(1./(self.A*sn + self.B))
		self.sigmaHat.push(1./(self.D*self.sTemp[-1] + self.E))
		sigman = self.sigmaHat.Data[-1]
		numerator = 0.5*4.*rho*(1. - rho)**2
		denominator = sn**2*(2. - rho)**2*sigman**3
		self.sTemp.append(sn + numerator/denominator)

	def parameterUpdate(self):
		xData = self.tauData.Data
		muHatData = self.muHat.Data
		self.uAtUpdateStep.append(numpy.mean(xData))
		#print xData, muHatData
		self.vAtUpdateStep.append(LamDelosmeVarianceFunction(xData,muHatData,self.tauIter))
		alpha = 1. - self.tauIter/self.La
		beta = 1. - self.tauIter/self.Lb
		L = len(self.uAtUpdateStep)
		f_1 = LamDelosmeSeriesFunction(numpy.matrix([1]*L),alpha)
		s = numpy.matrix(self.sTemp[::self.tauIter])
		soveru = numpy.divide(s,self.uAtUpdateStep)
		f_soveru = LamDelosmeSeriesFunction(soveru,alpha)
		f_s = LamDelosmeSeriesFunction(s,alpha)
		oneoveru = numpy.divide(1,self.uAtUpdateStep)
		f_1overu = LamDelosmeSeriesFunction(oneoveru,alpha)
		ssquared = numpy.power(s,2)
		f_ssquared = LamDelosmeSeriesFunction(ssquared,alpha)
		self.A = (f_1*f_soveru - f_s*f_1overu)/(f_1*f_ssquared - f_s*f_s)
		self.B = (f_1overu - self.A*f_s)/f_1
		g_1 = LamDelosmeSeriesFunction(numpy.matrix([1]*L),beta)
		soverv = numpy.divide(s,self.vAtUpdateStep)
		g_soverv = LamDelosmeSeriesFunction(soverv,beta)
		g_s = LamDelosmeSeriesFunction(s,beta)
		oneoverv = numpy.divide(1,self.vAtUpdateStep)
		g_1overv = LamDelosmeSeriesFunction(oneoverv,beta)
		g_ssquared = LamDelosmeSeriesFunction(ssquared,beta)
		self.D = (g_1*g_soverv - g_s*g_1overv)/(g_1*g_ssquared - g_s*g_s)
		self.E = (g_1overv - self.D*g_s)/g_1
