import numpy

def allResiduesWithinRadius(PDBFilePath,targetResidue,radius):
#For a given targetResidue in PDBFile, returns a list of other residue entries
#that have atoms within radius of the coordinate-wise center of mass of targetResidue.
#PDBFile is a file object for the PDB file you want to scan.
#targetResidue is a string containing the 3 or 4-digit sequence position number for
#the desired amino acid, e.g. '135' or '2374'.
#radius is a float, in angstroms.
	
	#Read in the data from the PDB file.
	PDBFile = open(PDBFilePath,'r')
	PDBData = PDBFile.readlines()
	PDBFile.close()

	#Find the lines that belong to the targetResidue.
	targetResidueData = []
	ATOMData = []
	for line in PDBData:
		if line[0:4] == 'ATOM':
			ATOMData.append(line)
	for line in ATOMData:
		scannedResiduePosition = line[22:26].strip()
		if scannedResiduePosition == targetResidue:
				targetResidueData.append(line)

	#Compute the target residue's coordinate center-of-mass (COM).
		xTarget = []
		yTarget = []
		zTarget = []
		for line in targetResidueData:
			xAtom = float(line[30:38])
			yAtom = float(line[38:46])
			zAtom = float(line[46:54])
			xTarget.append(xAtom)
			yTarget.append(yAtom)
			zTarget.append(zAtom)
		xCOM = numpy.mean(xTarget)
		yCOM = numpy.mean(yTarget)
		zCOM = numpy.mean(zTarget)
	
	#Scan each atom in the PDB file for nearness to the target residue's COM.
	intrudingResidues = []
	for line in ATOMData:
		residuePositionPDB = line[22:26]
		if residuePositionPDB in intrudingResidues:
			continue
		else:
			xAtomPDB = float(line[30:38])
			yAtomPDB = float(line[38:46])
			zAtomPDB = float(line[46:54])
			distance = numpy.sqrt((xAtomPDB - xCOM)**2 + \
								  (yAtomPDB - yCOM)**2 + \
								  (zAtomPDB - zCOM)**2)
		if distance <= radius:
			intrudingResidues.append(residuePositionPDB)
	return intrudingResidues
